/* dvipdft.cpp: run dvipdfm-gswin32-dvipdfm
   Copyright (C) 2000 Christian Schenk <cs@miktex.org>
   Time-stamp: "2000-09-08 23:50:21 mik"

   This file is part of dvipdm.

   dvipdfm is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   dvipdfm is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with dvipdfm; if not, write to the Free Software Foundation,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */

#include <cstdlib>
#include <cstdarg>
#include <string>
#include <direct.h>
#include <io.h>

#include "miktex.h"

using namespace std;
using namespace MiKTeX;

namespace {
  char szProgname[_MAX_PATH];
  char szTempFolder[_MAX_PATH];
  bool bMadeTempFolder;
}

/* _________________________________________________________________________

   FatalError
   _________________________________________________________________________ */

void
FatalError (const char *	fmt,
				...)
{
  fprintf (stderr, "%s: ", szProgname);
  va_list marker;
  va_start (marker, fmt);
  vfprintf (stderr, fmt, marker);
  va_end (marker);
  fputc ('\n', stderr);
  throw (1);
}

/* _________________________________________________________________________

   FatalFileError
   _________________________________________________________________________ */

void
FatalFileError (const char * filename)
{
  fprintf (stderr, "%s: ", szProgname);
  perror (filename);
  throw (1);
}

/* _________________________________________________________________________

   BadUsage
   _________________________________________________________________________ */

void
BadUsage ()
{
  FatalError ("no file name specified");
}

/* _________________________________________________________________________

   CreateTempFolder

   Prepare the temporary folder.  Remove it at exit, unless debugging.
   _________________________________________________________________________ */

void
CreateTempFolder (char * szPath)
{
  char szScratchFolder[_MAX_PATH];
  if (! GetTempFolder(szScratchFolder))
    FatalError ("no appropriate temp folder");

  size_t maxrounds = 5;
  while (maxrounds)
    {
      // create temp name
      UINT uUnique = (static_cast<double>(rand()) / RAND_MAX) * 0xffff;
      if (GetTempFileName(szScratchFolder, "dpt", uUnique, szPath) == 0)
	FatalError ("cannot perform GetTempFileName");
      if (_access(szPath, 0) != 0)
	break;
      maxrounds--;
    }
  if (maxrounds == 0)
    FatalError ("cannot create temp folder");
  if (_mkdir(szPath) != 0)
    FatalFileError (szPath);
}

/* _________________________________________________________________________

   dvipdft
   _________________________________________________________________________ */

void
dvipdft (int		argc,
	 char **	argv)
{
  GetExeName (szProgname);

  // Must have at least one argument (the DVI filename)
  if (argc == 1)
    {
      BadUsage ();
    }

  char szGSExePath[_MAX_PATH];
  if (! GSGetExe(szGSExePath))
    {
      FatalError ("cannot find gswin32c");
    }

  char szExePath[_MAX_PATH];
  if (! FindMiKTeXExecutable("dvipdfm", szExePath))
    {
      FatalError ("cannot find dvipdfm");
    }
  RemoveBlanksFromPathName (szExePath);

  CreateTempFolder (szTempFolder);
  bMadeTempFolder = true;

  // save original command-line options
  string strUserArgs;
  size_t i;
  for (i = 1; i < argc; ++i)
    {
      strUserArgs += ' ';
      strUserArgs += argv[i];
    }

  // loop over all arguments except the last one
  string strArgs;
  char szPdfFilename[_MAX_FNAME];
  szPdfFilename[0] = 0;
  for (i = 1; i < argc - 1; ++i)
    {
      strArgs += ' ';
      strArgs += argv[i];
      if (strcmp(argv[i], "-o") == 0)
	{
	  char drive[_MAX_DRIVE];
	  char dir[_MAX_DIR];
	  char fname[_MAX_FNAME];
	  _splitpath (argv[i+1], drive, dir, fname, 0);
	  _makepath (szPdfFilename, drive, dir, fname, 0);
	  strArgs += argv[i+1];
	  ++i;
	}
    }

  const char * pszUserFilename = argv[argc-1];
  if (szPdfFilename[0] == 0)
    strcpy (szPdfFilename, pszUserFilename);

  // run dvipdfm with the fastest options for the first pass
  string strCommandLine = szExePath;
  strCommandLine += strArgs;
  strCommandLine += " -e -z0 ";
  strCommandLine += pszUserFilename;
  DWORD dwExitCode = 0;
  if (! RunProcess(strCommandLine.c_str(), 0, 0, &dwExitCode, 0))
    {
      FatalError ("cannot perform %s", szExePath);
    }
  if (dwExitCode != 0)
    throw (static_cast<int>(dwExitCode));

  // run GhostScript to create thumbnails
  char szTempFilename[_MAX_PATH];
  _makepath (szTempFilename, 0, szTempFolder, szPdfFilename, "%d");
  strCommandLine = szGSExePath;
  strCommandLine += " -r10 -dNOPAUSE -dBATCH -sDEVICE=png256 -sOutputFile=";
  strCommandLine += szTempFilename;
  strCommandLine += ' ';
  strCommandLine += szPdfFilename;
  strCommandLine += ".pdf";
  if (! RunProcess(strCommandLine.c_str(), 0, 0, &dwExitCode, 0))
    {
      FatalError ("cannot perform %s", szGSExePath);
    }
  if (dwExitCode != 0)
    throw (static_cast<int>(dwExitCode));

  // run dvipdfm with the users specified options for the last pass
  strCommandLine = szExePath;
  strCommandLine += " -dt ";
  strCommandLine += strUserArgs;
  printf ("%s\n", strCommandLine.c_str());
  string MIKTEX_TEMP = "MIKTEX_TEMP=";
  MIKTEX_TEMP += szTempFolder;
  _putenv (MIKTEX_TEMP.c_str());
  if (! RunProcess(strCommandLine.c_str(), 0, 0, &dwExitCode, 0))
    {
      FatalError ("cannot perform %s", szExePath);
    }
  if (dwExitCode != 0)
    throw (static_cast<int>(dwExitCode));
}

/* _________________________________________________________________________

   main
   _________________________________________________________________________ */

int
main (int	argc,
      char **	argv)
{
  MiKTeX::MiKTeXInitialize ();
  bMadeTempFolder = false;
  int ret = 0;
  try
    {
      dvipdft (argc, argv);
    }
  catch (int code)
    {
      ret = code;
    }
  if (bMadeTempFolder)
    RemoveFolder (szTempFolder);
  MiKTeX::MiKTeXUninitialize ();
  return (ret);
}
